import requests

def send_markdown_message(token, content) -> bool:
    """
    发送markdown格式的企业微信通知
    @param token: 企业微信的token
    @param content: 通知内容
    @return: bool
    """
    resultBool = True
    if not token:
        raise Exception("你必须传入token的值")
    if not content:
        raise Exception("你必须传入content的值")

    url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={token}'
    headers = {"Content-Type": "application/json; charset=utf-8"}
    # logger.info(f'获取的发信内容: {content}')
    send_data = {
        "msgtype": "markdown",
        "markdown": {
            "content": content
        }
    }
    req = requests.post(url, json=send_data, headers=headers)
    result = req.json()
    if result['errcode'] != 0:
        resultBool = False
    return resultBool



from loguru  import logger
import pendulum
from pathlib import Path
from openpyxl import Workbook
from typing import List, Dict, Any, Tuple

def write_list_tuple_excel(datas: List[List[Tuple[Any]]], headers: List[str] = None, filename: str = None) -> None:
    """
    将列表元组写入到excel文件中
    datas: 包含多个数据列表的列表；每个数据列表作为一个 sheet 写入 Excel 文件
           eg. [[('argo', 'elk'),('argo2', 'elk2')], [(...), (...)]]
    headers: 表头列表,默认None,不写表头; ['命名空间', '项目名称']
    filename: 写入excel中的文件名,默认为None
    """
    # 判断文件是否已经存在,如果存在则删除
    if filename:
        if Path(filename).exists() or Path(f'{filename}.xlsx').exists():
            try:
                Path(filename).unlink()
                Path(f'{filename}.xlsx').unlink()
            except Exception:
                pass

    wb = Workbook()
    wb.encoding = 'utf-8'

    for index, data in enumerate(datas, start=1):
        ws = wb.create_sheet(title=f"Sheet{index}")
        if headers:
            ws.append(headers)
        for item in data:
            ws.append(item)
        # 设置单元格宽度
        for col in ws.columns:
            max_length = 0
            column = col[0].column_letter
            for cell in col:
                try:
                    if len(str(cell.value)) > max_length:
                        max_length = len(str(cell.value))
                except:
                    pass
            adjusted_width = (max_length + 2)
            ws.column_dimensions[column].width = adjusted_width

    # 删除默认的Sheet
    wb.remove(wb['Sheet'])

    # 自动添加后缀xlsx
    if filename and not str(filename).endswith('.xlsx'):
        filename = filename +  '.xlsx'

    if not filename:
        filename = f'write-{pendulum.now().to_datetime_string()}.xlsx'

    wb.save(filename)
    logger.success(f'写入文件成功,文件名: {filename}')

def write_list_dict_excel(datas: List[List[Dict[str, Any]]], headers: Dict[str, str]=None, filename: str=None):
    """
    将字典列表写入excel中
    datas: 包含多个数据字典列表的列表；每个数据字典列表作为一个 sheet 写入 Excel 文件
           eg. [[{'ns': 'argo', 'tp': 'elk'}, {'ns': 'argo2', 'tp': 'elk2'}], [{...}, {...}]]
    headers: 字典，每个键值对对应列名和标题；如果未提供，则根据第一个数据字典的键值自动生成; {'ns': '命名空间', 'tp': '类型'}
    filename: 响应头中的文件名,提供前端获取
    """
    wb = Workbook()
    wb.encoding = 'utf-8'

    for index, data in enumerate(datas, start=1):
        ws = wb.create_sheet(title=f"Sheet{index}")
        if headers and isinstance(headers, dict):
            header_values = [headers.get(key, key) for key in data[0].keys()]
            ws.append(header_values)
        else:
            ws.append(list(data[0].keys()))

        for item in data:
            ws.append(list(item.values()))

        # 设置单元格宽度
        for col in ws.columns:
            max_length = 0
            column = col[0].column_letter
            for cell in col:
                try:
                    if len(str(cell.value)) > max_length:
                        max_length = len(str(cell.value))
                except:
                    pass
            adjusted_width = (max_length + 2)
            ws.column_dimensions[column].width = adjusted_width

    # 删除默认的Sheet
    wb.remove(wb['Sheet'])

    # 自动添加后缀xlsx
    if filename and not str(filename).endswith('.xlsx'):
        filename = filename +  '.xlsx'

    if not filename:
        filename = f'write-{pendulum.now().to_datetime_string()}.xlsx'

    wb.save(filename)
    logger.success(f'写入文件成功,文件名: {filename}')



import os
from typing import List, Dict
from aliyun.log import LogClient, GetLogsRequest
from cryptography.fernet import Fernet

class SlsLogStore:
    def __init__(self, key=None):
        self._access_key_id, self._access_key_secret = SlsLogStore._get_access_key(key)

    @staticmethod
    def _get_access_key(key):
        encryption_account = {
            "access_key_id": 'gAAAAABmFlzfOJRPM9r8y37flIiJkM3TO6NrVdYK62cZGsqoEzymIzCZ8LbUdnkoPk5K8ibOo4GD-DPOos00B0ocaWcvkvsVfu-DtcSbxCJFkAOJVS0x9-8=',
            "access_key_secret": 'gAAAAABmFl0AY5ych1DeBtlpv2efMzqzrRaerEjOgB5H5cJKjlyWVMQ5e0KutYM7tBvHIxL3BUR1MqIqdkXY6ZF9Mpg30miCa2zKbkz2FPuQ9Z2EoiMx5vc='
        }
        if not key:
            key = os.environ.get('fernet_key')
            if not key:
                raise Exception("请先设置fernet_key")
        try:
            cipher_suite = Fernet(key.encode('utf-8'))
        except Exception as e:
            raise Exception(f"密钥格式不正确: {e}")

        access_key_id = cipher_suite.decrypt(encryption_account.get('access_key_id').encode('utf-8')).decode('utf-8')
        access_key_secret = cipher_suite.decrypt(encryption_account.get('access_key_secret').encode('utf-8')).decode('utf-8')

        return access_key_id, access_key_secret

    def _get_logs(self, tp, from_timestamp, to_timestamp, query) -> List[Dict]:
        result = []
        config = {
            "hz_log": {
                "endpoint": "cn-hangzhou.log.aliyuncs.com",
                "project_name": "hz-k8s",
                "logstore_name": "hz-k8s"
            },
            "hz_ingress": {
                "endpoint": "cn-hangzhou.log.aliyuncs.com",
                "project_name": "hz-k8s",
                "logstore_name": "nginx-ingress"
            },
            "usa_log": {
                "endpoint": "us-west-1.log.aliyuncs.com",
                "project_name": "usa-k8s",
                "logstore_name": "usa-log"
            },
            "usa_ingress": {
                "endpoint": "us-west-1.log.aliyuncs.com",
                "project_name": "usa-k8s",
                "logstore_name": "usa-ingress"
            }
        }
        endpoint, project_name, logstore_name = config.get(tp).values()
        client = LogClient(endpoint, self._access_key_id, self._access_key_secret)
        request = GetLogsRequest(project_name, logstore_name, from_timestamp, to_timestamp, query=query)
        response = client.get_logs(request)
        for log in response.get_logs():
            result.append(dict(log.contents))
        return result

    def _sls_config_res(self, tp, from_timestamp, to_timestamp, query) -> List[Dict]:
        try:
            result = self._get_logs(tp, from_timestamp, to_timestamp, query)
        except Exception as e:
            raise Exception(f"查询 {tp} 日志失败: {e}")
        return result

    def hz_ingress_log(self, from_timestamp, to_timestamp, query) -> List[Dict]:
        """
        查询杭州的ingress日志
        @param from_timestamp: 开始时间戳
        @param to_timestamp: 结束时间戳
        @param query: 查询条件
        @return: 查询结果(List类型)
        """
        return self._sls_config_res('hz_ingress', from_timestamp, to_timestamp, query)

    def hz_service_log(self, from_timestamp, to_timestamp, query) -> List[Dict]:
        """
        查询杭州的服务日志
        @param from_timestamp: 开始时间戳
        @param to_timestamp: 结束时间戳
        @param query: 查询条件
        @return: 查询结果(List类型)
        """
        return self._sls_config_res('hz_log', from_timestamp, to_timestamp, query)

    def usa_ingress_log(self, from_timestamp, to_timestamp, query) -> List[Dict]:
        """
        查询美国的ingress日志
        @param from_timestamp: 开始时间戳
        @param to_timestamp: 结束时间戳
        @param query: 查询条件
        @return: 查询结果(List类型)
        """
        return self._sls_config_res('usa_ingress', from_timestamp, to_timestamp, query)

    def usa_service_log(self, from_timestamp, to_timestamp, query) -> List[Dict]:
        """
        查询美国的服务日志
        @param from_timestamp: 开始时间戳
        @param to_timestamp: 结束时间戳
        @param query: 查询条件
        @return: 查询结果(List类型)
        """
        return self._sls_config_res('usa_log', from_timestamp, to_timestamp, query)
